@@ -1,47 +1,29 @@
 # -*- coding: UTF-8 -*-
 
-"""Y's API for Google Trends.
+"""
+=========================
+Y's API for Google Trends
+=========================
 
-This (yagt) is a reimplementation of pyGTrends by Sal Uryasev
+yagt improves the pyGTrends tool written by Sal Uryasev
 (http://www.juiceanalytics.com/openjuice/programmatic-google-trends-api/).  The
-authentication and connection of yagt are borrowed from Sal Uryasev's
-pyGTrends.  In addition to provide a more Pythonic way to manipulate the
-downloaded data, yagt also give you a simple command line interface (CLI) to 
-grab data from Google Trends and save to a csv file.
+authentication and connection of remains similar.  In addition to provide a
+more Pythonic way to manipulate the downloaded data, yagt also give you a
+simple command line interface (CLI) to grab data from Google Trends and save to
+a csv file.
 
 After install the module, you can access the simple CLI by using::
 
-  $ python -m yagt <query1> <query2> <-u username> [options]
+  $ python -m yagt <query1> <query2> [-u username] [other options]
 
 The output will be saved to ``result.csv'' by default.  For more information
-about the options, please use ``python -m yagt -h'' for the help page."""
+about the options, please use ``python -m yagt -h'' for the help page.  See
+``LICENSE.txt'' for license agreement.
+"""
 
-__license__ = """Copyright (c) 2009, Yung-Yu Chen
-All rights reserved.
+__version__ = '0.3'
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
-    * Redistributions of source code must retain the above copyright notice,
-      this list of conditions and the following disclaimer.
-    * Redistributions in binary form must reproduce the above copyright notice,
-      this list of conditions and the following disclaimer in the documentation
-      and/or other materials provided with the distribution.
-    * Neither the name of Yung-Yu Chen nor the names of its contributors may be
-      used to endorse or promote products derived from this software without
-      specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
-FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
-OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."""
-
-__version__ = '0.2.3'
+__description__ = """Y's API for Google Trends."""
 
 class TrendsDict(dict):
     """
@@ -93,8 +75,10 @@ def write_csv(rows, filename):
     @type filename: str
     @return: nothing.
     """
+    from codecs import BOM_UTF8
     from csv import writer
     fobj = open(filename, 'w')
+    fobj.write(BOM_UTF8)
     writer = writer(fobj, lineterminator='\n')
     writer.writerows(rows)
     fobj.close()
@@ -113,7 +97,8 @@ class Query(object):
         ('Accept', 'text/plain'),
     ]
 
-    def __init__(self, username, password, do_data=False, do_dict=True):
+    def __init__(self, username, password,
+      do_data=False, do_raw=False, do_dict=True):
         """
         Take the authenticating information and connect to Google Trends.
         Create the opener.
@@ -124,6 +109,8 @@ class Query(object):
         @type password: str
         @keyword do_data: return downloaded data.
         @type do_data: bool
+        @keyword do_raw: return also raw data.
+        @type do_raw: bool
         @keyword do_dict: return dictionary processed from download data.
         @type do_dict: bool
         """
@@ -132,8 +119,9 @@ class Query(object):
         from cookielib import CookieJar
         from urllib import urlencode
         from urllib2 import build_opener, HTTPCookieProcessor
-        self.do_data = do_data
         self.do_dict = do_dict
+        self.do_data = do_data
+        self.do_raw = do_raw
         # request authentication.
         ## prepare connection parameters.
         alphanums = ascii_letters + digits
@@ -160,7 +148,8 @@ class Query(object):
         
     def __call__(self, *args, **kw):
         """
-        Download the report of a specific set of query keywords.
+        Download the report of a specific set of query keywords.  All the query
+        arguments should be unicode objects.
 
         @keyword date: a string for the month/year to query.  The format for 
             year is 'YYYY'.  The format for month is 'YYYY-M'.  It can also be
@@ -187,7 +176,7 @@ class Query(object):
         from urllib import urlencode
         # request information.
         params = dict(
-            q=','.join(args),
+            q=','.join(arg.encode('utf-8') for arg in args),
             date='all',
             geo='all',
             graph='all_csv',
@@ -195,37 +184,34 @@ class Query(object):
             scale=0,
             sa='N',
         )
-        params.update(kw)
+        for key in params.keys():   # don't pull in unused keys.
+            params[key] = kw.get(key, params[key])
         params = urlencode(params)
         raw = self.opener.open('http://www.google.com/trends/viz?'+params).read()
         # clean up unwanted characters.
-        data = self._clean(raw)
+        data = raw.decode('utf-16le')
+        data = data.replace('\r\n', '\n').strip().encode('utf-8')
         if data == 'You must be signed in to export data from Google Trends':
             raise ValueError, data
         # craft returns.
         if self.do_dict:
             result = TrendsDict(data=data)
             if self.do_data:
-                return result, data
+                if self.do_raw:
+                    return result, data, raw
+                else:
+                    return result, data
             else:
                 return result
         else:
             if self.do_data:
-                return data
+                if self.do_raw:
+                    return data, raw
+                else:
+                    return data
             else:
                 return
 
-    @staticmethod
-    def _clean(raw):
-        """
-        @param raw: raw data.
-        @type raw: str
-        @return: cleaned data.
-        @rtype: str
-        """
-        data = raw[2::2]
-        return data.replace('\r\n', '\n').strip()
-
 class CmdParam(object):
     """
     Command line parameters.
@@ -233,12 +219,13 @@ class CmdParam(object):
     def __init__(self):
         from optparse import OptionParser, OptionGroup
         op = OptionParser(
-            usage='usage: %prog <term1> <term2> ... <-u username> [options]',
+            usage='usage: %prog <term1> <term2> ... '
+                '[-u username] [other options]',
             version='%%prog %s' % __version__,
         )
         opg = OptionGroup(op, 'Authentication Information')
         opg.add_option('-u', action='store', type='string',
-            dest='username',
+            dest='username', default='',
             help='The username of a Google Account.',
         )
         op.add_option_group(opg)
@@ -255,35 +242,64 @@ class CmdParam(object):
         )
         op.add_option_group(opg)
         opg = OptionGroup(op, 'Output')
+        opg.add_option('-p', action='store',
+            dest='parts', default='day,week',
+            help='Specify the part/section to output.  '
+                'Use comma to separate multiple parts.',
+        )
         opg.add_option('-f', action='store',
             dest='filename', default='result.csv',
             help='Output file name.',
         )
+        opg.add_option('-k', action='store_true',
+            dest='append_key', default=False,
+            help='Flag to append the name of section to the main output file '
+                'name.  If set, only the last valid specified section will be '
+                'saved, since previous ones will be overridden.  Default is '
+                'False.',
+        )
         op.add_option_group(opg)
         # set to self.
         self.op = op
         self.options, self.arguments = self.op.parse_args()
 
-def main():
+def cmdline():
     """
     Command line frontend.
 
     @return: nothing.
     """
+    import sys
+    import os
     import getpass
     # get the command line options and arguments.
     clps = CmdParam()
     opts, args = clps.options, clps.arguments
-    # get the password.
+    # get the username/password.
+    username = opts.username
+    if not username:
+        username = raw_input('Username: ')
     password = getpass.getpass('Password: ')
     # connect to Google.
-    query = Query(opts.username, password)
+    query = Query(username, password)
     # prepare and make a query to Google Trends.
     kw = dict(date=opts.date)
+    args = [arg.decode(sys.stdin.encoding) for arg in args]
     result = query(*args, **kw)
-    # save to file.
-    key = 'day' if 'day' in result else 'week'
-    write_csv(result[key], opts.filename)
+    # save to files.
+    parts = opts.parts.split(',')
+    for key in parts:
+        rows = result.get(key, None)
+        if rows == None:
+            continue
+        filename = opts.filename
+        if opts.append_key:
+            main, ext = os.path.splitext(filename)
+            filename = main + '_%s'%key + ext
+        write_csv(rows, filename)
+
+def main():
+    cmdline()
 
 if __name__ == '__main__':
     main()
